# frozen_string_literal: true

require 'memoist'
require 'faraday'
require 'better-faraday'

module Peatio
  module Decredcoin
    class Client
      Error = Class.new(StandardError)
      ConnectionError = Class.new(Error)

      class ResponseError < Error
        def initialize(code, msg)
          @code = code
          @msg = msg
        end

        def message
          "#{@msg} (#{@code})"
        end
      end

      extend Memoist

      def initialize(endpoint, is_secure = false)
        @json_rpc_endpoint = URI.parse(endpoint)
        @is_secure = is_secure
      end

      def json_rpc(method, params = [])
        response = connection.post \
          '/',
          { jsonrpc: '2.0', method: method, params: params }.to_json,
          { 'Accept' => 'application/json',
            'Content-Type' => 'application/json' }
        response.assert_2xx!
        response = JSON.parse(response.body)
        response['error'].tap do |e|
          raise ResponseError.new(e['code'], e['message']) if e
        end
        response.fetch('result')
      rescue Faraday::Error => e
        raise ConnectionError, e
      end

      private

      def connection
        user, pass = [@json_rpc_endpoint.user, @json_rpc_endpoint.password]
        @connection ||= Faraday.new(@json_rpc_endpoint) do |f|
          f.adapter :net_http, pool_size: 5
          f.ssl[:verify] = @is_secure
        end.tap do |connection|
          connection.basic_auth(user, pass) unless user.blank?
        end
      end
    end
  end
end
